@@ -15,7 +15,7 @@ import numpy as np
 from matplotlib.widgets import Slider
 from MJOLNIR import _tools
 from MJOLNIR._interactiveSettings import Viewer3DSettings, States, cut1DHolder
-from MJOLNIR.Data.DraggableShapes import clearBoxes, DraggableCircle,DraggableRectangle, prepareInteractiveCuttingView3D,\
+from MJOLNIR._interactiveSettings import clearBoxes, DraggableCircle,DraggableRectangle, prepareInteractiveCuttingView3D,\
     extractCut1DPropertiesRectangle, extractCut1DPropertiesCircle
 import functools
 
@@ -25,7 +25,7 @@ pythonSubVersion = sys.version_info[1]
 class Viewer3D(object):  
     @_tools.KwargChecker(include=[_tools.MPLKwargs])
     def __init__(self,Data,bins,axis=2, log=False ,ax = None, grid = False, adjustable=True, outputFunction=print, 
-                 cmap=None, CurratAxeBraggList=None,Ei=None,EfLimits=None, dataset = None, cut1DFunctionRectangle=None, cut1DFunctionCircle = None, **kwargs):#pragma: no cover
+                 cmap=None, CurratAxeBraggList=None, plotCurratAxe=False,Ei=None,EfLimits=None, dataset = None, cut1DFunctionRectangle=None, cut1DFunctionCircle = None, **kwargs):#pragma: no cover
         """3 dimensional viewing object generating interactive Matplotlib figure. 
         Keeps track of all the different plotting functions and variables in order to allow the user to change between different slicing modes and to scroll through the data in an interactive way.
 
@@ -49,6 +49,8 @@ class Viewer3D(object):
             - outputFunction (function): Function called on output string (default print)
 
             - CurratAxeBraggList (list): List of Bragg peaks for which Currat-Axe spurions are to be calculated, [[H1,K1,L1],[H2,K2,L2],..] (default None)
+
+            - plotCurratAxe (bool): Flag to determine whether or not to plot the Currat-Axe spurions
             
             - Ei (list): List of incoming energies (default None)
             
@@ -84,16 +86,18 @@ class Viewer3D(object):
         self.currentData = None
         
 
-        self.plotCurratAxe = False # Set to false but will change to True when correct list is created
+        self.plotCurratAxe = plotCurratAxe # Set to false but will change to True when correct list is created
         self._CurratAxeBraggList = None
         if len(Data)==4: # If data is provided as I, norm, mon, normcount
             with warnings.catch_warnings() as w:
+                warnings.simplefilter("ignore")
                 self.Data = np.divide(Data[0]*Data[3],Data[1]*Data[2])
             
             self.Counts,self.Monitor,self.Normalization,self.NormCounts = Data
             self.allData = True
         elif len(Data) == 2: # From bin3D with no normalization or monitor
             with warnings.catch_warnings() as w:
+                warnings.simplefilter("ignore")
                 self.Data = np.divide(Data[0],Data[1])
             self.allData = False
         else:
@@ -118,7 +122,7 @@ class Viewer3D(object):
             self.grid = False
             self.gridZOrder = 0
 
-        if ax is None:
+        if ax is None: # TODO: REDO with actual correct axes!
             self.figure = plt.figure()
             self.ax = plt.subplot(gs[0])#self.figure.add_subplot(111)
             self.xlabel = r'Qx [$A^{-1}$]'
@@ -177,12 +181,12 @@ class Viewer3D(object):
             ax.set_navigate(True)
         self.value = 0
         self.figure.subplots_adjust(bottom=0.25)
-        self.cmap = cmap # Update to accomedate deprication warning
+        self.cmap = cmap # Update to accommodate deprecation warning
         self.value = 0
         self.setAxis(2)
         # Set up interactive generation of 1DCuts
 
-        def cut1DFunctionRectangleDefault(self,dr):
+        def cut1DFunctionRectangleDefault(self,dr):# pragma: no cover
             global cut1DHolder
             parameters = extractCut1DPropertiesRectangle(dr.rect,self.ax.sample)
             step = self.dQE[self.axis]
@@ -194,7 +198,7 @@ class Viewer3D(object):
             cut1DHolder.append([self.ds.plotCut1D(**parameters,Emin=EMin,Emax=EMax)])
 
 
-        def cut1DFunctionCircleDefault(self,dr):
+        def cut1DFunctionCircleDefault(self,dr):# pragma: no cover
             global cut1DHolder
             parameters = extractCut1DPropertiesCircle(dr.circ,self.ax.sample)
             parameters['E1'] = self.ds.energy.min()
@@ -247,13 +251,20 @@ class Viewer3D(object):
         #self.imcbaxes = self.figure.add_axes([0.0, 0.2, 0.2, 0.7])
         #self.im = self.ax.imshow(self.masked_array[:,:,self.value].T,cmap=self.cmap,extent=[self.X[0],self.X[-1],self.Y[0],self.Y[-1]],origin='lower')
         if self.shading=='flat':
+            self.ax.grid(False)
             self.im = self.ax.pcolormesh(self.X[:,:,0].T,self.Y[:,:,0].T,self.masked_array[:,:,self.value].T,zorder=10,shading=self.shading,cmap=cmap)
         elif self.shading=='gouraud':  # pragma: no cover
             XX = 0.5*(self.X[:-1,:-1,self.value]+self.X[1:,1:,self.value]).T
             YY = 0.5*(self.Y[:-1,:-1,self.value]+self.Y[1:,1:,self.value]).T
+            self.ax.grid(False)
             self.im = self.ax.pcolormesh(XX,YY,self.masked_array[:,:,self.value].T,zorder=10,shading=self.shading,cmap=cmap) # ,vmin=1e-6,vmax=6e-6
         else:
             raise AttributeError('Did not understand shading {}.'.format(self.shading))
+        if self.grid:
+            self.ax.grid(self.grid,zorder=self.gridZOrder)
+        else:
+            self.ax.grid(self.grid)
+
         self._caxis = self.im.get_clim()
         self.figpos = [0.125,0.25,0.63,0.63]#self.ax.get_position()
         
@@ -276,15 +287,23 @@ class Viewer3D(object):
         self.dQE = [dQx,dQy,ddE]
 
         self.resolution = [0.05,0.05]
-        if not self.EfLimits is None:
-            self.Ef = np.arange(*self.EfLimits,self.dQE[2])
-            
-            self.CurratAxeBraggList = CurratAxeBraggList
+        
                 
 
         self.Energy_slider.set_val(self.value)
+        if self.rlu:
+            #self.cid = self.figure.canvas.mpl_connect('button_press_event', lambda event: eventdecorator(onclick,self,event,outputFunction=outputFunction))
+            self.axRLU.onClickFunction = lambda event: eventdecorator(onclick,self,event,outputFunction=outputFunction,extra='axRLU')
+            # self.axRLU._button_press_event = self.axRLU.get_figure().canvas.mpl_connect('button_press_event', self.axRLU.onClickFunction)
 
-        self.cid = self.figure.canvas.mpl_connect('button_press_event', lambda event: eventdecorator(onclick,self,event,outputFunction=outputFunction))
+            self.axQxE.onClickFunction = lambda event: eventdecorator(onclick,self,event,outputFunction=outputFunction,extra='axQxE')
+            # self.axQxE._button_press_event = self.axQxE.get_figure().canvas.mpl_connect('button_press_event', self.axQxE.onClickFunction)
+
+            self.axQyE.onClickFunction = lambda event: eventdecorator(onclick,self,event,outputFunction=outputFunction,extra='axQyE')
+            # self.axQyE._button_press_event = self.axQyE.get_figure().canvas.mpl_connect('button_press_event', self.axQyE.onClickFunction)
+        else:
+            self.figure.canvas.mpl_connect('key_press_event',lambda event: onkeypress(event, self) )
+        
         
         try:
             maxVal = np.nanmax(self.masked_array[np.isfinite(self.masked_array)])
@@ -301,6 +320,11 @@ class Viewer3D(object):
         self.setPlane(1)
         self.setPlane(0)
 
+        if not self.EfLimits is None:
+            self.Ef = np.arange(*self.EfLimits,self.dQE[2])
+            
+            self.CurratAxeBraggList = CurratAxeBraggList
+
         if adjustable and pythonVersion>2 and pythonSubVersion>5:
 
             ax_cmin = plt.axes([0.87, 0.1, 0.05, 0.7])
@@ -354,8 +378,12 @@ class Viewer3D(object):
             self.im.set_array(self.emptyData) # Set data in current im to a fully masked data set
         if axis==2:
             if self.rlu:
+                if hasattr(self.ax,'_button_press_event'):
+                    self.ax.get_figure().canvas.mpl_disconnect(self.ax._button_press_event)
                 self.figure.delaxes(self.ax)
                 self.ax = self.figure.add_axes(self._axes[axis])
+                if hasattr(self.ax,'onClickFunction'):
+                    self.ax._button_press_event = self.ax.get_figure().canvas.mpl_connect('button_press_event', self.ax.onClickFunction)
             else:
                 self.ax.set_xlabel(self.xlabel)
                 self.ax.set_ylabel(self.ylabel)
@@ -363,8 +391,12 @@ class Viewer3D(object):
             label = self.zlabel#self.ax.get_ylabel
         elif axis==1:  # pragma: no cover
             if self.rlu:
+                if hasattr(self.ax,'_button_press_event'):
+                    self.ax.get_figure().canvas.mpl_disconnect(self.ax._button_press_event)
                 self.figure.delaxes(self.ax)
                 self.ax = self.figure.add_axes(self._axes[axis])
+                if hasattr(self.ax,'onClickFunction'):
+                    self.ax._button_press_event = self.ax.get_figure().canvas.mpl_connect('button_press_event', self.ax.onClickFunction)
             else:
                 self.ax.set_xlabel(self.xlabel)
                 self.ax.set_ylabel(self.zlabel)
@@ -372,8 +404,12 @@ class Viewer3D(object):
             label =  self.ylabel#self.ax.get_ylabel
         elif axis==0:  # pragma: no cover
             if self.rlu:
+                if hasattr(self.ax,'_button_press_event'):
+                    self.ax.get_figure().canvas.mpl_disconnect(self.ax._button_press_event)
                 self.figure.delaxes(self.ax)
                 self.ax = self.figure.add_axes(self._axes[axis])
+                if hasattr(self.ax,'onClickFunction'):
+                    self.ax._button_press_event = self.ax.get_figure().canvas.mpl_connect('button_press_event', self.ax.onClickFunction)
             else:
                 self.ax.set_xlabel(self.ylabel)
                 self.ax.set_ylabel(self.zlabel)
@@ -495,7 +531,7 @@ class Viewer3D(object):
             self.im.set_array(tempData)
             self._axesChanged = False
         else:
-            self.im.set_array(self.masked_array[:,:,self.value].T.flatten())
+            self.im.set_array(self.masked_array[:,:,int(self.value)].T.flatten())
         
             
         if not self.CurratAxeBraggList is None and not self.Ei is None and self.plotCurratAxe is True and self.rlu is True:
@@ -569,7 +605,7 @@ def eventdecorator(function,self,event,*args,**kwargs):# pragma: no cover
         return function(self,event.xdata,event.ydata,*args,**kwargs)
 
 
-def onclick(self,x,y,returnText=False, outputFunction=print): # pragma: no cover
+def onclick(self,x,y,returnText=False, outputFunction=print,extra=None): # pragma: no cover
     idz = self.value
     axis = self.axis
 
@@ -602,7 +638,8 @@ def onclick(self,x,y,returnText=False, outputFunction=print): # pragma: no cover
         Mon = self.Monitor[ID[0],ID[1],ID[2]]
         NC = self.NormCounts[ID[0],ID[1],ID[2]]
         printString+=', Cts = {:}, Norm = {:.3f}, Mon = {:d}, NormCount = {:d}'.format(cts,Norm,int(Mon),NC)
-
+    if not extra is None:
+        printString+=extra
     if returnText:
         return printString
     else:
@@ -627,12 +664,17 @@ def onkeypress(event,self): # pragma: no cover
             reloadslider(self,0)
             del self.im
             #self.currentData = None
+            self.ax.grid(False)
             if self.shading=='flat':
                 self.im = self.ax.pcolormesh(self.X[:,:,0].T,self.Y[:,:,0].T,self.masked_array[:,:,self.value].T,zorder=10,shading=self.shading,cmap=self.cmap)
             elif self.shading=='gouraud':
                 self.im = self.ax.pcolormesh(0.5*(self.X[:-1,:-1,0]+self.X[1:,1:,0]).T,0.5*(self.Y[:-1,:-1,0]+self.Y[1:,1:,0]).T,self.masked_array[:,:,self.value].T,zorder=10,shading=self.shading,cmap=self.cmap) # ,vmin=1e-6,vmax=6e-6
             else:
                 raise AttributeError('Did not understand shading {}.'.format(self.shading))
+            if self.grid:
+                self.ax.grid(self.grid,zorder=self.gridZOrder)
+            else:
+                self.ax.grid(self.grid)
             self.im.set_clim(self.caxis)
             self.Energy_slider.set_val(0)
             self.plot()
@@ -643,12 +685,17 @@ def onkeypress(event,self): # pragma: no cover
             reloadslider(self,1)
             del self.im
             #self.currentData = None
+            self.ax.grid(False)
             if self.shading=='flat':
                 self.im = self.ax.pcolormesh(self.X[:,:,0].T,self.Y[:,:,0].T,self.masked_array[:,:,self.value].T,zorder=10,shading=self.shading,cmap=self.cmap)
             elif self.shading=='gouraud':
                 self.im = self.ax.pcolormesh(0.5*(self.X[:-1,:-1]+self.X[1:,:1:]).T,0.5*(self.Y[:-1,-1]+self.Y[1:,1:]).T,self.masked_array[:,:,self.value].T,zorder=10,shading=self.shading,cmap=self.cmap) # ,vmin=1e-6,vmax=6e-6
             else:
                 raise AttributeError('Did not understand shading {}.'.format(self.shading))
+            if self.grid:
+                self.ax.grid(self.grid,zorder=self.gridZOrder)
+            else:
+                self.ax.grid(self.grid)
             self.im.set_clim(self.caxis)
             self.Energy_slider.set_val(0)
             self.plot()
@@ -659,6 +706,7 @@ def onkeypress(event,self): # pragma: no cover
             reloadslider(self,2)
             del self.im
             #self.currentData = None
+            self.ax.grid(False)
             if self.shading=='flat':
                 self.im = self.ax.pcolormesh(self.X[:,:,0].T,self.Y[:,:,0].T,self.masked_array[:,:,self.value].T,zorder=10,shading=self.shading,cmap=self.cmap)
             elif self.shading=='gouraud':
@@ -667,6 +715,10 @@ def onkeypress(event,self): # pragma: no cover
                 self.im = self.ax.pcolormesh(XX,YY,self.masked_array[:,:,self.value].T,zorder=10,shading=self.shading,cmap=self.cmap) # ,vmin=1e-6,vmax=6e-6
             else:
                 raise AttributeError('Did not understand shading {}.'.format(self.shading))
+            if self.grid:
+                self.ax.grid(self.grid,zorder=self.gridZOrder)
+            else:
+                self.ax.grid(self.grid)
             self.im.set_clim(self.caxis)
             self.Energy_slider.set_val(0)
             self.plot()
@@ -674,7 +726,7 @@ def onkeypress(event,self): # pragma: no cover
             self.ax.set_ylim([np.min(self.Y),np.max(self.Y)])
 
 
-def reloadslider(self,axis): # pragma: no cover
+def reloadslider(self,axis):
     self.setAxis(axis)
     self.Energy_slider.set_val(0)
     self.Energy_slider.label.remove()
@@ -827,7 +879,7 @@ def addColorbarSliders(self,c_min,c_max,c_minval,c_maxval,ax_cmin,ax_cmax,log=Tr
     
     fig.savefig = savefig
 
-def cut1DFunctionDefault(self,dr):
+def cut1DFunctionDefault(self,dr):# pragma: no cover
     global cut1DHolder
     parameters = extractCut1DProperties(dr.rect,self.ax.sample)
     step = self.dQE[self.axis]
