@@ -6,9 +6,8 @@ import numpy as np
 from MJOLNIR.Geometry import GeometryConcept,Analyser,Detector,Wedge
 from MJOLNIR import _tools
 import MJOLNIR
-from MJOLNIR import TasUBlibDEG as TasUBlib
-from MJOLNIR.TasUBlibDEG import cosd,sind
-from MJOLNIR.Data import Sample,RLUAxes
+from MJOLNIR import TasUBlibDEG
+from MJOLNIR.Data import RLUAxes
 import warnings
 import matplotlib.pyplot as plt
 from mpl_toolkits.mplot3d import Axes3D
@@ -17,6 +16,8 @@ from scipy.stats import norm
 import h5py as hdf
 import datetime
 import pytest
+from MJOLNIR.Geometry.eck_Flipped import get_E, get_scattering_angle,get_mono_angle,get_angle_ki_Q,get_angle_kf_Q,calc_eck
+from MJOLNIR import _interactiveSettings
 
 NumberOfSigmas= 3.0 # Defining the active area of a peak on a detector as \pm n*sigma
 predictionInstrumentSupport = ['CAMEA','MultiFLEXX','Bambus'] # Instrument supported in prediction function
@@ -394,7 +395,10 @@ class Instrument(GeometryConcept.GeometryConcept):
                 
                 monitor = np.array(VanFile.get('entry/control/data')) # Extract monitor count
                 # Divide data with corresponding monitor by reshaping it correctly
-                Data = np.array(VanFileInstrument.get('detector/counts')).transpose(2,0,1).astype(float)/monitor[np.newaxis,:,np.newaxis]
+                intensities = VanFileInstrument.get('detector/counts')
+                if intensities is None:
+                    intensities = VanFileInstrument.get('detector/data')
+                Data = np.array(intensities).transpose(2,0,1).astype(float)/monitor[np.newaxis,:,np.newaxis]
                 if sampleMass != None: # A sample mass has been proivided
 
                     sampleMolarMass = _tools.calculateMolarMass(sample,formulaUnitsPerUnitCell)
@@ -410,6 +414,13 @@ class Instrument(GeometryConcept.GeometryConcept):
                     Data[:,:,:100]=0
 
                 Ei = np.array(VanFileInstrument.get('monochromator/energy')).astype(float)
+
+                # Due to motor positioning error by too small steps, one scan could be unordered along energy!
+                sortIDXEi = np.argsort(Ei)
+                
+                Ei = Ei[sortIDXEi]
+                Data = Data[:,sortIDXEi]
+
                 analysers = 8
                 pixels = self.wedges[0].detectors[0].pixels
                 detectors = len(self.A4[0])*len(self.A4)
@@ -627,9 +638,11 @@ class Instrument(GeometryConcept.GeometryConcept):
                                     binPixelData = binPixelData[BotId:TopId]
                                     EiLocal = Ei[BotId:TopId]
                                     Bg = np.min(binPixelData[[0,-1]])
-                                    guess = np.array([np.max(binPixelData), ECenter,0.08],dtype=float)
+                                    guess = np.array([np.max(binPixelData), ECenter,0.08],dtype=float) # Bg
+                                    fitFunc = lambda x,A,mu,sigma:Gaussian(x,A,mu,sigma,0.0)
                                     try:
-                                        res = scipy.optimize.curve_fit(lambda x,A,mu,sigma:Gaussian(x,A,mu,sigma,0.0),EiLocal,binPixelData.astype(float),p0=guess)
+                                        res = scipy.optimize.curve_fit(fitFunc,EiLocal,binPixelData.astype(float),p0=guess)
+                                        
                                         
                                     except: # pragma: no cover
                                         if not os.path.exists(savelocation+'/{}_pixels'.format(detpixels)):
@@ -638,12 +651,12 @@ class Instrument(GeometryConcept.GeometryConcept):
                                             plt.ioff
                                         plt.figure()
                                         plt.scatter(EiLocal,binPixelData)
-                                        plt.plot(Ei,Gaussian(Ei,*guess,0.0))
+                                        plt.plot(Ei,fitFunc(Ei,*guess))
                                     
                                         plt.savefig(savelocation+'/{}_pixels/Detector{}_{}.png'.format(detpixels,i,k),format='png',dpi=150)
                                         plt.close()
-
-                                    fittedParameters[i,j,k]=np.concatenate([res[0],[0.0]])
+                                    
+                                    fittedParameters[i,j,k]=list(res[0])+[0.0]
                                     fittedParameters[i,j,k,0] *= np.sqrt(2*np.pi)*fittedParameters[i,j,k,2] #np.sum(binPixelData) # Use integrated intensity as amplitude 29-07-2020 JL
                                      
                                     if plot: # pragma: no cover
@@ -755,7 +768,7 @@ class Instrument(GeometryConcept.GeometryConcept):
 
                     fitParameters.append(fittedParameters)
                     activePixelRanges.append(np.array(activePixelDetector))
-                    tableString = 'Normalization for {} pixel(s) using VanData {} and A4Data{}\nPerformed {}\nDetector,Energy,Pixel,IntegratedIntensity,Center,Width,Background,lowerBin,upperBin,A4Offset\n'.format(detpixels,Vanadiumdatafile,A4datafile,datetime.datetime.now())
+                    tableString = 'Normalization for {} pixel(s) using VanData {} and A4Data{}{}\nPerformed {}\nDetector,Energy,Pixel,IntegratedIntensity,Center,Width,Background,lowerBin,upperBin,A4Offset\n'.format(detpixels,Vanadiumdatafile,A4datafile,(sampleMass!=0)*', using Vanadium mass {}'.format(sampleMass),datetime.datetime.now())
                     for i in range(len(fittedParameters)):
                         for j in range(len(fittedParameters[i])):
                             for k in range(len(fittedParameters[i][j])):
@@ -1168,7 +1181,7 @@ def convertToHDF(fileName,title,sample,fname,CalibrationFile=None,pixels=1024,ce
             tth.append(2.*th)
         return theta,tth
 
-    def storeScanData(entry,data,a3,a4,ei,rotation_angle_zero=0.0,polar_angle_offset=0.0):
+    def storeScanData(entry,data,a3,a4,ei,rotation_angle_zero=0.0,polar_angle_offset=0.0,dir='.'):
         nxdata = entry.create_group('data')
         nxdata.attrs['NX_class'] = np.string_('NXdata')
         
@@ -1244,13 +1257,26 @@ def convertToHDF(fileName,title,sample,fname,CalibrationFile=None,pixels=1024,ce
         #dset.attrs['units'] = np.string_('counts')
         
         
-        makeMonitor(entry,Numpoints)
+        makeMonitor(entry,Numpoints,dir)
         entry.create_dataset('scancommand',data=[np.string_(scanType)])
         entry.create_dataset('scanvars',data=scanvars)
-    def makeMonitor(entry,Numpoints):
+    def makeMonitor(entry,Numpoints,dir):
+        
+        ## read monitor if slit monitor exists
+
+        # Check if slit monitor exists
+        if os.path.exists(os.path.join(dir,'0','SlitMonitor.dat')):
+            mons = []
+            for  i in range(Numpoints):
+                dat = np.loadtxt(os.path.join(dir,str(i),'SlitMonitor.dat'))
+                totalCount = dat[:int(len(dat)/3)].sum()
+                mons.append(totalCount)
+        else:
+            mons = [10000]*Numpoints
+
         control = entry.create_group('control')
         control.attrs['NX_class'] = np.string_('NXmonitor')
-        mons = [10000]*Numpoints
+        
         control.create_dataset('data',data=mons,dtype='int32')
         dset = control.create_dataset('preset',(1,),dtype='int32')
         dset[0] = 10000
@@ -1306,37 +1332,37 @@ def convertToHDF(fileName,title,sample,fname,CalibrationFile=None,pixels=1024,ce
         import os
         Numpoints = sum([os.path.isdir(fname+'/'+i) for i in os.listdir(fname)])
         data,a3,a4,ei = readScanData(fname,Numpoints,factor=factor,pixels=pixels,detectors=detectors)
-        storeScanData(entry,data,a3,a4,ei,rotation_angle_zero=rotation_angle_zero,polar_angle_offset=polar_angle_offset)
+        storeScanData(entry,data,a3,a4,ei,rotation_angle_zero=rotation_angle_zero,polar_angle_offset=polar_angle_offset,dir=fname)
         
 
-def converterToA3A4(Qx,Qy, Ei,Ef,A3Off=0.0,A4Sign=-1):
+def converterToA3A4(Qx,Qy, Ei,Ef,A3Off=0.0,A4Sign=-1): # pragma: no cover
     Qx = np.asarray(Qx)
     Qy = np.asarray(Qy)
 
     QC = np.array([Qx,Qy])
     q = np.linalg.norm(QC)
 
-    U1V = np.array([Qx.flatten(),Qy.flatten(),0.0],dtype=float)
+    U1V = np.array([Qx.flatten(),Qy.flatten(),np.zeros_like(Qx.flatten())],dtype=float)
 
     U1V/=np.linalg.norm(U1V)
     U2V = np.array([0.0,0.0,1.0],dtype=float)
     
     
-    TV = TasUBlib.buildTVMatrix(U1V, U2V)
+    TV = TasUBlibDEG.buildTVMatrix(U1V, U2V)
     R = np.linalg.inv(TV)
     
     ss = 1.0
     
     cossgl = np.sqrt(R[0,0]*R[0,0]+R[1,0]*R[1,0])
-    om = TasUBlib.arctan2d(R[1,0]/cossgl, R[0,0]/cossgl)
+    om = TasUBlibDEG.arctan2d(R[1,0]/cossgl, R[0,0]/cossgl)
     
     ki = np.sqrt(Ei)*_tools.factorsqrtEK
     kf = np.sqrt(Ef)*_tools.factorsqrtEK
     
     cos2t =(ki**2 + kf**2 - q**2) / (2. * np.abs(ki) * np.abs(kf))
     
-    A4 = ss*TasUBlib.arccosd(cos2t)
-    theta = TasUBlib.calcTheta(ki, kf, A4)
+    A4 = ss*TasUBlibDEG.arccosd(cos2t)
+    theta = TasUBlibDEG.calcTheta(ki, kf, A4)
     A3 = -om + np.sign(A4Sign)*ss*theta + A3Off
     return A3,np.sign(A4Sign)*A4
 
@@ -1347,7 +1373,7 @@ def converterToQxQy(A3,A4,Ei,Ef):
     kf = np.sqrt(Ef)*_tools.factorsqrtEK
 
     r = [0,0,0,A3,A4,0.0,0.0,Ei,Ef]
-    QV = TasUBlib.calcTasUVectorFromAngles(r)
+    QV = TasUBlibDEG.calcTasUVectorFromAngles(r)
     q = np.sqrt(ki**2 +kf**2-
         2. *ki *kf * np.cos(np.deg2rad(A4)))
 
@@ -1385,24 +1411,40 @@ def prediction(A3Start,A3Stop,A3Steps,A4Positions,Ei,Cell,r1,r2,points=False,out
         - instrument (string): Instrument for which the prediction is to be made (default CAMEA)
 
     """
+    from MJOLNIR.Data import Sample
     s = Sample.Sample(*Cell,projectionVector1=r1,projectionVector2=r2)
  
     class simpleDataSet():
-        def __init__(self,sample):
+        def __init__(self,sample,Ei):
             self.sample = [sample]
+            self.Ei = np.array([Ei])
+
+        def __len__(self):
+            return 1
+
+        def FindEi(self,deltaE):
+            return self.Ei
      
     if outputFunction is None:
         outputFunction = print
 
-    t = simpleDataSet(s)
+    t = simpleDataSet(s,Ei)
     fig = plt.figure(figsize=(16,11))
     
     if instrument == 'CAMEA':
-        calib = np.loadtxt(MJOLNIR.__CAMEANormalization__,delimiter=',',skiprows=3)
+        calib = np.loadtxt(MJOLNIR.__CAMEANormalizationBinning1__,delimiter=',',skiprows=3)
         detectors = 104
         wedges = 8
         energies = 8
-        Ax = [RLUAxes.createRLUAxes(t,figure=fig,ids=(3,3,i+1)) for i in range(9)]
+        Ax = []
+        for i in range(9):
+            ax = RLUAxes.createQAxis(t,withoutOnClick=True,figure=fig,ids=(3,3,i+1))
+
+            #ax.type = 'QPlane'
+            ax.ds = t
+            #ax = _interactiveSettings.setupModes(ax)
+            Ax.append(ax)
+        
 
     elif instrument == 'MultiFLEXX':
         calib = np.loadtxt(MJOLNIR.__multiFLEXXNormalization__,delimiter=',',skiprows=1)
@@ -1410,7 +1452,7 @@ def prediction(A3Start,A3Stop,A3Steps,A4Positions,Ei,Cell,r1,r2,points=False,out
         wedges = 31
         energies = 5
         A4Width = 0.5 # Pad the prediction in A4 with this value (otherwise only lines will be shown)
-        Ax = [RLUAxes.createRLUAxes(t,figure=fig,ids=(2,3,i+1)) for i in range(6)]
+        Ax = [RLUAxes.createQAxis(t,withoutOnClick=True,figure=fig,ids=(2,3,i+1)) for i in range(6)]
 
     elif instrument == 'Bambus':
         calib = np.loadtxt(MJOLNIR.__bambusNormalization__,delimiter=',',skiprows=1)
@@ -1418,10 +1460,10 @@ def prediction(A3Start,A3Stop,A3Steps,A4Positions,Ei,Cell,r1,r2,points=False,out
         wedges = 20
         energies = 5
         A4Width = 0.5 # Pad the prediction in A4 with this value (otherwise only lines will be shown)
-        Ax = [RLUAxes.createRLUAxes(t,figure=fig,ids=(2,3,i+1)) for i in range(6)]
+        Ax = [RLUAxes.createQAxis(t,withoutOnClick=True,figure=fig,ids=(2,3,i+1)) for i in range(6)]
 
     else:
-        raise AttributeError('Privded instrumnt "{}" not understood'.format(instrument))
+        raise AttributeError('Provided instrument "{}" not understood'.format(instrument))
     
     A4Instrument = calib[:,-1].reshape(detectors,energies)
     
@@ -1438,12 +1480,14 @@ def prediction(A3Start,A3Stop,A3Steps,A4Positions,Ei,Cell,r1,r2,points=False,out
     startColor = np.array([0.83921569, 0.15294118, 0.15686275, 0.5])
     diff = (endColor-startColor)/float(energies-1)
 
-    def format_axes_func(self,x,y,Ei,Ef,A3Off=s.theta,A4Sign=-1.0):
+    def format_axes_func(self,x,y,Ei,Ef,A3Off=s.theta,A4Sign=-1.0):# pragma: no cover
         A3,A4 = converterToA3A4(x,y,Ei,Ef,-A3Off,A4Sign=A4Sign)
         return ax.sample.format_coord(x,y)+', A3 = {:.2f} deg, A4 = {:.2f} deg, Ef = {:.2f}'.format(A3,A4,Ef)
     
     for i,(ax,Ef,A4) in enumerate(zip(Ax,EfInstrument.reshape(detectors,energies).T,A4Instrument.reshape(detectors,energies).T)):
         ax.Ef = np.nanmean(Ef)
+        ax.EMin = Ei-ax.Ef
+        ax.EMax = Ei-ax.Ef
         
         for A4Position in A4Positions:
             if points:
@@ -1497,7 +1541,7 @@ def prediction(A3Start,A3Stop,A3Steps,A4Positions,Ei,Cell,r1,r2,points=False,out
         Ax[4].format_coord = lambda x,y: format_axes_func(Ax[4],x,y,Ei,np.nanmean(EfInstrument,axis=0)[4],-s.theta,A4Sign=A4Positions[0])
     
 
-    def onclick(event,axes,outputFunction):
+    def onclick(event,axes,outputFunction):# pragma: no cover
         for ax in axes:
             if ax.in_axes(event):
                 try:
@@ -1517,4 +1561,163 @@ def prediction(A3Start,A3Stop,A3Steps,A4Positions,Ei,Cell,r1,r2,points=False,out
                 break
     fig.canvas.mpl_connect('button_press_event', lambda event: onclick(event,Ax,outputFunction=outputFunction))
     fig.tight_layout()
-    return Ax
\ No newline at end of file
+    return Ax
+
+def calculateResoultionMatrix(position,Ei,Ef,sample = None, rlu=True,binning = 8,A3Off=0.0, instrument='CAMEA'):
+    """calculate Resolution matrix at position in units if 1/AA,1/AA,1/AA,meV
+    
+    Args:
+
+        - position (list): Position used for calculation, either Qx,Qy or (H,K,L)
+
+        - Ei (float): Incoming energy in meV
+
+        - Ef (float): Outgoing energy in meV
+
+    Kwargs:
+
+        - sample (MJOLNIR.Sample): Sample used to convert between HKL and QxQy (default None -> QxQy)
+
+        - rlu (bool): Whether or not to recalculate position from HKL to QxQy (default True)
+
+        - binning (int): Binning to be used when calculating resolution for CAMEA (default 8)
+
+        - instrument (str): Instrument used to calculate resolution (default CAMEA)
+    """
+
+
+    # Calculate to QxQy if not provided
+    if rlu or sample is None:
+        #qe = np.concatenate([Position,[Ei,Ef]])
+        Qx,Qy = sample.calculateHKLToQxQy(*position)
+        
+    else:
+        Qx,Qy = position
+    
+    
+    if instrument == 'CAMEA': #CAMEA
+        if not binning>0 and binning <9:
+            raise AttributeError('Provided binning "{:}" not understood. Must be an integer between 1 and 8.'.format(binning))
+        fileName = getattr(MJOLNIR,'__CAMEANormalizationBinning{0}__'.format(binning))
+        calib = np.loadtxt(fileName,delimiter=',',skiprows=3)
+    else:
+        NotImplementedError('Currently, only calculations for CAMEA are implemented....')
+    ## find detector corresponding to Ef and A4
+    Efs = calib[:,3:7].reshape(104,binning*8,4)[:,:,1].mean(axis=0)
+    EfIdx = np.argmin(np.abs(Ef-Efs))
+
+    
+    if instrument == 'CAMEA': #CAMEA
+        distancesSampleAnalyzer=np.array([0.9300,0.9939,1.0569,1.1195,1.1827,1.2456,1.3098,1.3747])
+    else:
+        raise NotImplementedError('Currently, only calculations for CAMEA are implemented....')
+    dist = distancesSampleAnalyzer[int(np.floor(EfIdx/8))]
+    
+    cm2A = 1e8
+    min2rad = 1./ 60. / 180.*np.pi
+    rad2deg = 180. / np.pi
+    
+    d_mono = 3.355
+    d_ana = 3.355
+    
+    ki = np.sqrt(Ei)*TasUBlibDEG.factorsqrtEK
+    kf = np.sqrt(Ef)*TasUBlibDEG.factorsqrtEK
+    Q = np.linalg.norm([Qx,Qy])
+    
+    
+    E = get_E(ki, kf)
+    
+    A3,A4 = converterToA3A4(Qx,Qy,Ei,Ef,A3Off=A3Off)
+    sc_senses = [ 1., -1., 1.]
+    
+    ## Create the parameter list needed for the eck_Flipped function
+    params = {
+        # scattering triangle
+        "ki" : ki, "kf" : kf, "E" : E, "Q" : Q,
+    
+        # angles
+        "twotheta" : get_scattering_angle(ki, kf, Q),
+        "thetam" : get_mono_angle(ki, d_mono),
+        "thetaa" : get_mono_angle(kf, d_ana),
+        "angle_ki_Q" : get_angle_ki_Q(ki, kf, Q),
+        "angle_kf_Q" : get_angle_kf_Q(ki, kf, Q),
+    
+        # scattering senses
+        "mono_sense" : sc_senses[0],
+        "sample_sense" : sc_senses[1],
+        "ana_sense" : sc_senses[2],
+    
+        # distances
+        "dist_src_mono" : 120. * cm2A,
+        "dist_mono_sample" : 160. * cm2A,
+        "dist_sample_ana" : dist*100. * cm2A,
+        "dist_ana_det" : 70. * cm2A,
+    
+        # component sizes
+        "src_w" : 2. * cm2A,
+        "src_h" : 3. * cm2A,
+        "mono_w" : 26. * cm2A,
+        "mono_h" : 17. * cm2A,
+        "det_w" : 1.27 * cm2A,
+        "det_h" : 1. * cm2A,
+        "ana_w" : 5. * cm2A,
+        "ana_h" : 5. * cm2A,
+    
+        # focusing
+        "mono_curvh" : 1.,
+        "mono_curvv" : 1.,
+        "ana_curvh" : 1.,
+        "ana_curvv" : 0.,
+        "mono_is_optimally_curved_h" : True,
+        "mono_is_optimally_curved_v" : True,
+        "ana_is_optimally_curved_h" : True,
+        "ana_is_optimally_curved_v" : False,
+        "mono_is_curved_h" : True,
+        "mono_is_curved_v" : True,
+        "ana_is_curved_h" : True,
+        "ana_is_curved_v" : False,
+    
+        # collimation
+        "coll_h_pre_mono" : 9999. *min2rad,
+        "coll_v_pre_mono" : 9999. *min2rad,
+        "coll_h_pre_sample" : 9999. *min2rad,
+        "coll_v_pre_sample" : 9999. *min2rad,
+        "coll_h_post_sample" : 9999. *min2rad,
+        "coll_v_post_sample" : 60. *min2rad,
+        "coll_h_post_ana" : 9999. *min2rad,
+        "coll_v_post_ana" : 9999. *min2rad,
+    
+        # guide
+        "use_guide" : True,
+        "guide_div_h" : 120. *min2rad,
+        "guide_div_v" : 120. *min2rad,
+    
+        # mosaics
+        "mono_mosaic" : 42. *min2rad,
+        "mono_mosaic_v" : 42. *min2rad,
+        "ana_mosaic" : 42. *min2rad,
+        "ana_mosaic_v" : 42. *min2rad,
+    
+        # crystal reflectivities
+        # TODO, so far always 1
+        "dmono_refl" : 1.,
+        "dana_effic" : 1.,
+    
+        # off-center scattering
+        # WARNING: while this is calculated, it is not yet considered in the ellipse plots
+        "pos_x" : 0. * cm2A,
+        "pos_y" : 0. * cm2A,
+        "pos_z" : 0 * cm2A,
+    }
+    
+    
+    rescalc = calc_eck(params)
+    M = rescalc['reso']
+
+    # Rotate to Qx, Qy with A3
+    # 4D rotation matrix but only rotation in Qx,Qy
+    R = np.eye(4)
+    R[:2,:2] = _tools.Rot(-A3).reshape(2,2)
+
+    MRotated = np.dot(R,np.dot(M,R.T))
+    return MRotated
